//************************************************************************
// T6963.C
// LowLevel Routinen fr LCD-Displays mit T6963
// Getestet bis 8MHz.
//
// Hier fr Winstar WG24064A
//
// Ausgelegt bisher nur auf diesen Displaytyp
// Alle Variablen zur Koordinatenbergabe sind 8Bit lang.
// Das reicht fr Displays mit 128x128, 240x64 und 240x128.
//
// Die Gre kann man in Display.h ndern !
//
// Der Nullpunkt des Displays liegt oben,links. Fr unten rechts
// mssen dann zustzliche Umrechnungen stattfinden.
//
// Mit SetPixel hat man den Grundstein fr alle
// erdenklichen Zeichenroutinen:
// Linien, Rechtecke, Kreise, Fllen, Muster
//
// Was witziges nebenbei: Wenn mein Display von einer Leuchtstofflampe
// Energiesparlampe beleuchtet wird, flimmert es unangenehm !
// Stroboskopeffekt.
// Da hat Winstar eine schlechte Quarzfrequenz gewhlt.
// Hlt man eine Taschenlampe drauf ist es vllig ruhig.
//
// hk@holger-klabunde.de
// http://www.holger-klabunde.de
// Compiler WinAVR 3.4.6
//************************************************************************

#include <avr/io.h>
//#include <sig-avr.h>

#include "protos.h"
#include "T6963.h"

//##########################################################
void CheckBusy(void)
//##########################################################
{
 unsigned char by;

 DATA_IN(); //DatenPort auf Eingang zum lesen
 CTRL();    //Status Register lesen

 do         //Schleife Status Bits checken
  {
   CE_OFF();
   RD_OFF();
   NOP();    //Ein Waitstate fr 8 MHZ
   NOP();    //plus noch einen fr 16 MHZ
   by=READ_DATA(); //Status lesen
   RD_ON();
   CE_ON();
  }while((by&3) != 3); //Solange STA0 oder STA1 = 0

 DATA();   //Zurck auf Display Daten schreiben
}

//##########################################################
void WriteCTRL(unsigned char dat)
//##########################################################
{
 DATA_OUT(); //DatenPort auf Ausgang zum schreiben
 CTRL();     //Commando schreiben vorbereiten
 WRITE_DATA(dat); //Daten zum Commando

 CE_OFF();
 WR_OFF();
 NOP();    //Ein Waitstate fr 8 MHZ
 NOP();    //plus noch einen fr 16 MHZ
 WR_ON();
 CE_ON();

 DATA();   //Zurck auf DisplayDaten schreiben
}

//##########################################################
//Schreibt ein Datenbyte und ein Kommandobyte
void WriteData1(unsigned char dat, unsigned char command)
{
 WriteData(dat); //Daten schreiben
 WriteCommand(command); //Kommando schreiben
}

//##########################################################
//Schreibt zwei Datenbytes und ein Kommandobyte
void WriteData2(unsigned int dat, unsigned char command)
{
 WriteData((unsigned char)(dat&0xFF)); //Daten schreiben
 WriteData((unsigned char)(dat>>8)); //Daten schreiben
 WriteCommand(command); //Kommando schreiben
}

//##########################################################
//Datenbyte schreiben
void WriteData(unsigned char dat)
{
 CheckBusy(); //StatusBits STA0,1 checken

 DATA_OUT(); //DatenPort auf Ausgang zum schreiben
 DATA();     //DisplayDaten schreiben vorbereiten
 WRITE_DATA(dat); //Daten auf den Port

 CE_OFF();
 WR_OFF();
 NOP();    //Ein Waitstate fr 8 MHZ
 NOP();    //plus noch einen fr 16 MHZ
 WR_ON();
 CE_ON();
}

//##########################################################
//Kommandobyte schreiben
void WriteCommand(unsigned char command)
{
 CheckBusy(); //StatusBits STA0,1 checken
 WriteCTRL(command); //Kommando schreiben
}

//##########################################################
//Pixel auf dem Display setzen
//mode=0; Pixel lschen
//mode=1; Pixel setzen
void SetPixel(unsigned int xpos, unsigned int ypos, unsigned char mode)
{
 unsigned int adress;
 unsigned char tmp,pixel;

//Statt Displaywerte als Grenzen zu nehmen nur checken ob in eine
//gltige DisplayRAM Adresse geschrieben wird !
// if(xpos>=LCD_WIDTH) return; //Punkt auerhalb der Anzeige
// if(ypos>=LCD_HEIGHT) return;

//Position im Display-RAM setzen
 adress=GRAPH_AREA; //Start of Grafikarea

//anhand von xpos und ypos ermitteln welches Byte
//gelesen/geschrieben werden soll
 adress+=((LCD_WIDTH/PIXPERBYTE)*ypos); //Anzahl Zeilen*Zeilenbreite
 adress+=xpos/PIXPERBYTE;               //plus X-Position

 if(adress>G_END1) return;

 if(mode==0) pixel=0xF0; //Kommando Pixel lschen
 else pixel=0xF8;        //Kommando Pixel setzen

//Die Reihenfolge der Pixel ist im Byte genau umgekehrt !
 tmp=(PIXPERBYTE-1)-(xpos%PIXPERBYTE);
 pixel|=tmp; //Position des Pixels im Byte setzen

//Set address pointer
 WriteData((unsigned char)(adress&0xFF)); //Daten schreiben
 WriteData((unsigned char)(adress>>8)); //Daten schreiben
 WriteCommand(0x24); //Kommando schreiben
 WriteCommand(pixel); //Befehl mit gewnschtem Pixel schreiben
}

//##########################################################
//Gewnschte Koordinate auf dem Display einstellen
void SetPosition(unsigned char xpos, unsigned char ypos)
{
 unsigned int adress;

 if(xpos>=LCD_WIDTH) return; //Punkt auerhalb der Anzeige
 if(ypos>=LCD_HEIGHT) return;

 adress=GRAPH_AREA; //Start of Grafikarea

//anhand von xpos und ypos ermitteln welches Byte
//gelesen/geschrieben werden soll
 adress+=((LCD_WIDTH/PIXPERBYTE)*ypos);
 adress+=xpos/PIXPERBYTE;

//Set address pointer
 WriteData((unsigned char)(adress&0xFF)); //Daten schreiben

 WriteData((unsigned char)(adress>>8)); //Daten schreiben

 WriteCommand(0x24); //Kommando schreiben
}

//##########################################################
void InitDisplay(void)
{
 INIT_CONTROL_PINS(); //WR,CE,RD,C/D auf Ausgnge
  
 WR_ON();
 CE_ON();
 RD_ON();
 DATA();

 DATA_OUT(); //DatenPort auf Ausgang zum schreiben
 
//Write text home address=0x0000
 WriteData2(TEXT_AREA,0x40);

//Write graphic home address
 WriteData2(GRAPH_AREA,0x42);

//Text area column set= Setze Textspalten 
//Write text area address=001FH = 30 Spalten
//Set Low bit= MaxColumn/8
 WriteData2(LCD_WIDTH/CHARWIDTH,0x41);

//Graphics area column set in Bytes !!!
//Write graphic area address=001FH = 30 Bytes
//Set Low bit= MaxColumn/8
 WriteData2(LCD_WIDTH/PIXPERBYTE,0x43);

//set display mode = OR
 WriteCommand(0x80);

//Text and graphic display !!
 WriteCommand(0x9C);

 ClearScreen(); //Lscht das Text UND Graphic RAM !
}

//##########################################################
// Zeichen von Windows nach T6963 konvertieren
unsigned char ConvertText(unsigned char ch)
{
 unsigned char tmp;

 tmp=ch;

//Die case-Werte sind bezogen auf den SystemFont !!!
 switch(ch)
  {
   case 196: tmp=0x6E; break; // 
   case 228: tmp=0x64; break; // 
   case 214: tmp=0x79; break; // 
   case 246: tmp=0x74; break; // 
   case 220: tmp=0x7A; break; // 
   case 252: tmp=0x61; break; // 
   case 223: tmp=223; break; // 
   default: tmp-=0x20; break;
  }

 return tmp;
}

//##########################################################
// Text horizontal schreiben
void LCDText(char *txt, unsigned char xpos, unsigned char ypos)
{
 char ps;

 WriteData2(xpos+(LCD_WIDTH/CHARWIDTH)*ypos,0x24);

 while((ps = *txt)) //Solange Zeichen grer 0x00
  {
   txt++; //Nchstes Zeichen
   if (ps== 0) break; //Raus wenn Zeichen gleich 0
   ps=ConvertText(ps); //bersetzungstabelle aufrufen
   WriteData1(ps,0xC0); //Write ps, Auto Inkrement
  }
}

//##########################################################
// Kompletten Bildschirm lschen
void ClearScreen(void)
{
 unsigned char j;
 int i;

//Write text home address
//Set address pointer = Text home
 WriteData2(TEXT_AREA,0x24);

//ClearText RAM
//Witzigerweise bleibt der Inhalt des Text RAM
//ohne Stromversorgung noch recht lange erhalten ;) 
 for(i=0; i<(LCD_WIDTH/CHARWIDTH * LCD_HEIGHT/8); i++) WriteData1(0x00,0xC0);

 WriteData2(GRAPH_AREA,0x24); //An Position 0:0

 for(j=0; j<LCD_HEIGHT; j++)
  {
   for(i=0; i<(LCD_WIDTH/PIXPERBYTE); i++) WriteData1(0x00,0xC0);
  }

}

//##########################################################
// Kompletten Bildschirm fllen
void FillScreen(void)
{
 unsigned char i,j;

 WriteData2(GRAPH_AREA,0x24); //An Position 0:0

 for(j=0; j<LCD_HEIGHT; j++)
  {
   for(i=0; i<(LCD_WIDTH/PIXPERBYTE); i++) WriteData1(0xFF,0xC0);
  }
}
