//************************************************************************
// T6963.H
//
// Define LCD Size and I/O-Hardware Pins
//
// hk@holger-klabunde.de
// http://www.holger-klabunde.de
// 05.03.2006
//************************************************************************
#ifndef t6963_h
#define t6963_h

#define LCD_WIDTH	240 //display width
#define LCD_HEIGHT	64  //display height

#define CHARWIDTH     	6	  //bits per character
#define PIXPERBYTE    	CHARWIDTH //write CHARWIDTH pixel per byte

#define BYTES_PER_ROW   LCD_WIDTH/PIXPERBYTE     // how many bytes per row on screen
#define DISPLAYBYTES    BYTES_PER_ROW*LCD_HEIGHT // how many bytes for a screen

#define G_BASE  0x0200            	// base address of first screen
#define G_END  	G_BASE+DISPLAYBYTES-1   // end adress of first screen
#define G_BASE1 G_BASE+DISPLAYBYTES     // base address of 2nd screen
#define G_END1 	G_BASE1+DISPLAYBYTES-1  // end adress of 2nd screen


#define TEXT_AREA     0 	//begin of textarea
#define GRAPH_AREA    G_BASE    //begin of graphicsarea

#define NOP() asm volatile ("nop" ::)
//WinAVR 3.4.3 setzt die Bitsetzbefehle sbi(),cbi() ins Assembler-Listing fr diese
//beiden #defines ein ! Also kein Grund Inline Assembler zu benutzen.
#define sbi(portn, bitn) ((portn)|=(1<<(bitn)))
#define cbi(portn, bitn) ((portn)&=~(1<<(bitn)))
//#define sbi(portn, bitn) asm volatile("sbi %0, %1" : : "I" (_SFR_IO_ADDR(portn)), "I" ((uint8_t)(bitn)))
//#define cbi(portn, bitn) asm volatile("cbi %0, %1" : : "I" (_SFR_IO_ADDR(portn)), "I" ((uint8_t)(bitn)))

//hardware dependent macros
//here for a Atmel AVR/ATMega processor
//PORTC is data port

#define INIT_CONTROL_PINS() { DDRD=0xFF; DDRA=0xFF; } //control pins to output

#define DATA_IN() DDRC=0x00;  	  // set io-pins to inputs
#define DATA_OUT() DDRC=0xFF;  	  // set io-pins to outputs
#define READ_DATA() PINC	  // read PINC, ! not ! PORTC !
#define WRITE_DATA(a)  PORTC=(a); // write to data port

#define WR_ON() sbi(PORTA,6);
#define WR_OFF() cbi(PORTA,6);

#define RD_ON() sbi(PORTA,7);
#define RD_OFF() cbi(PORTA,7);

#define CE_ON() sbi(PORTD,4);
#define CE_OFF() cbi(PORTD,4);

#define CTRL() sbi(PORTD,5);
#define DATA() cbi(PORTD,5);

//Prototypes
extern unsigned char ReadDisplay(void);
extern void WriteData1(unsigned char, unsigned char);
extern void WriteData2(unsigned int, unsigned char);
extern void WriteData(unsigned char dat);
extern void WriteCommand(unsigned char command);
extern void SetPixel(unsigned int xpos, unsigned int ypos, unsigned char mode);
extern unsigned char GetPixel(unsigned char xpos, unsigned char ypos);
extern void SetPosition(unsigned char xpos, unsigned char ypos);
extern void InitDisplay(void);
extern void ClearScreen(void);
extern void FillScreen(void);
extern unsigned char ConvertText(unsigned char ch);
extern void LCDText(char *txt, unsigned char xpos, unsigned char ypos);

#endif // t6963_h
